/*
 *	@(#)axv.c	1.1 (UWM) 9/8/86
 */

#include "axv.h"
#if NAXV > 0
/*
 * AD/DA Data conversion AXV interface -- Jeff Dieringer
 */

/*
 * The AXV is Digital's Qbus analog to digital/ digital to analog
 * to digital conversion device. This driver 
 * converts a stream of data in real time on a ISI 4.2bsd machine. 
 *
 * The AXV does not have a data buffer and can only transfer a single
 * word at a time. It may driven with interrupts or polling
 * loop and a read may be program or (external or internal)
 * clock initiated. The maximum sampling or processing rate for both
 * analog to digital and digital to analog conversion is 25k/sec.
 * 
 * The read and write routines require an external clock to initiate
 * each read or write. The ISI can not transfer the data from kernel
 * space to user space as fast as the input signal is being converted, so
 * one large buffer is compiled in. There is an ioctl option for
 * sending a high signal to one of the DAC's when the buffer
 * is full. If the source of the analog signal is able to
 * stop when receiving a high signal, then this may use to read
 * or write a stream of data longer than  the buffer.
 *
 * The maximum recommended sampling rate for polling is 8k/sec.
 * After this the data degrades and if pushed higher, the interrupt
 * stack over flows and the machine hangs. With the polling method
 * the system can easily handle the AXV advertised maximum conversion
 * rate of 25k samples /sec., but of course, no other jobs can be done.
 */
#include "../h/ioctl.h"
#include "../h/param.h"
#include "../h/errno.h"
#include "../h/buf.h"
#include "../h/uio.h"

#include "../is68kdev/qbvar.h"
#include "../is68kdev/axvreg.h"

#define AXVWAITPRI	(PZERO+8)
#define AXVUNIT(dev) 	(minor(dev))
#define	DELAY(n)	{ register int N = (n); while (--N > 0); }
#define CSRMASK		0x3fff

/* Driver status bits */
#define AXV_BUSY		01
#define AXV_OPEN		02
#define AXV_POLL		04
#define AXV_READ		010
#define AXV_RXREADY		020

/*
 * Definition of driver for the auto-configuration routines
*/
int	axvprobe(),	/* Routine to look for an AXV on the bus */ 
	axvattach(), 	/* Dummy attach routine */
	axvslave(),	/* Dummy slave routine */
	axvintr();	/* Handle receiver interrupts */

/*
 * Place to hold device specific information (one per each AXV)
 */
struct qb_device *axvdinfo[NAXV];

/*
 * The standard address you would expect to find a AXV
 */
u_short *axvstd[] = { (u_short *) 017770400, 0 };

/*
 * Definitions of the auto-configuration routines for ionconf.c
 */
struct qb_driver AXVdriver = { 
	axvprobe, 
	axvslave, 
	axvattach, 
	axvstd, 
	"axv", 
	axvdinfo, 
	"AXV"
};

/*
 * Current state of driver for each AXV device
 */
struct axv_softc {
	int		axv_trw;		/* Total read or written */ 
	int		axv_ctrw;	/* Current total read or written*/
	short int 	axv_softcsr;	/* Current gain, and input chan */
	short int	*axv_dac;	/* Current physical output channel */
	short int	axv_bigbuf[BIGBUF];
	short int 	axv_state;	/* state of driver */
					/*         set		not set */
					/* bit 0 - busy		not busy */
					/* bit 1 - open		not open */
					/* bit 2 - poll		interrupt */
					/* bit 3 - read op	write op */
					/* bit 4 - start op 	normal */
					/* bit 5-15 unused */
} axv[NAXV];


extern int cvec;


/*-------------------------------------------------------------------*/
/*
 * Called at boot up time to see if the physical device is really there.
 * The incoming parameter reg is from the csr field of a axv-11 device
 * listed in the site dependant configuration file. 
 *
 * This function sets the interrupt enable and start a sample
 * bits in the csd of the AXV11-c. If the device is there, the
 * external variable cvec is reset in a trap routine. The return
 * value is used to make sure the address space of the 
 * registers of the device do not overlap with other devices.
 */
axvprobe(reg)
	caddr_t reg;
	{
	register struct axvdevice *axvaddr = (struct axvdevice *) reg;

	axvaddr->axv_csr = AXV_IENABLE | AXV_START;
	DELAY(40000);
	axvaddr->axv_csr = 0;
	return (sizeof (struct axvdevice));
	}
/*-------------------------------------------------------------------*/
/*
 * Dummy slave routine. Actually the address in the structure
 * AXVdriver should be set to 0, but some systems puke on this.
 */
axvslave()
	{
	return(1);
	}
/*-------------------------------------------------------------------*/
/*
 * Dummy routine to attach AXV to the system. 
 */
axvattach(qi)
	struct qb_device *qi;
	{
	return;
	}
/*-------------------------------------------------------------------*/
/* 
 * First test if the device is alive and not already opened, and if
 * not, then set the default settings.
 */
axvopen(dev)
	dev_t dev;
	{
	register struct axv_softc *axvp;
	register struct axvdevice *axvaddr =
	    (struct axvdevice *) axvdinfo[AXVUNIT(dev)]->qi_mi->qm_addr;
	register struct qb_device *qi;
	
	/*
	 * Make sure the device exists and is not already open. The AXV
	 * is allowed only to be opened once.
	 */
	if (AXVUNIT(dev) >= NAXV || ((axvp = &axv[AXVUNIT(dev)])->axv_state & AXV_OPEN) ||
	    (qi = axvdinfo[AXVUNIT(dev)]) == 0 || qi->qi_alive == 0)
		return (ENXIO);
	/*
	 * Set the default parameters.
	 */
	axvp->axv_trw = 0;	/* Set upper index of data buffer to zero */
	axvp->axv_ctrw = 0;	/* Set lower index of data buffer to zero */
	axvp->axv_state = AXV_OPEN | AXV_POLL;	
				/* Set state to not busy */
				/* Set it open */
				/* Set driver to polling */
				/* Set to read for int. routine */
				/* Set to no start/stop signal transmit */
	axvp->axv_softcsr = 0;	/* Set soft csr to zero. This implies
				   the input channel is set to 0, and
				   the gain is set to +- 10 volts. */
	
				/* Set output channel to zero */
	axvaddr = (struct axvdevice *) qi->qi_mi->qm_addr;
	axvp->axv_dac = &(axvaddr->axv_dacA);
	
	return (0);
	}
/*-------------------------------------------------------------------*/
/*
 * Set the device to closed and to
 * to not busy.
 */
axvclose(dev)
	dev_t dev;
	{
	struct axv_softc *axvp = &axv[AXVUNIT(dev)];
	
	axvp->axv_state = 0;
	}
/*-------------------------------------------------------------------*/
/*
 * AXV ioctl calls include:
 * ADIOGSTATUS - get status of software and hardware
 * ADIOSCHAN   - set input channel
 * ADIOSGAIN   - set gain of input and output
 * ADIOSDAC    - set output port (DAC)
 * ADIOTEST    - polling loop test to read and immediately write data
 * ADIOGETW    - read a single input. read initated with software.
 * ADIOPUTW    - write a single word of data to DAC
 * ADIORXREADY - send start/stop signal to DAC B and data to DAC A
 */
axvioctl(dev, cmd, data, flag)
	dev_t dev;
	register caddr_t data;
	{
	register struct axvdevice *axvaddr =
	    (struct axvdevice *) axvdinfo[AXVUNIT(dev)]->qi_mi->qm_addr;
	register struct qb_device *qi = axvdinfo[AXVUNIT(dev)];
	register struct axv_softc *axvp;
	register int index0 = 0;
	register int index1 = 0;
	register short int *dac;
	struct axvstatus *axvs;
	
	axvp = &axv[AXVUNIT(dev)];
	/*
	 * If a device is open or not openned, a user can get its status.
	 */
	if ( cmd == ADIOGSTATUS )
		{
		axvs = (struct axvstatus *) data;
		axvs->axv_chan = (int)((axvp->axv_softcsr & 0x0f00) >> 8);
		if ( &(axvaddr->axv_dacA) == axvp->axv_dac )
			axvs->axv_dac  = 0;
		else
			axvs->axv_dac  = 1;
		axvs->axv_trw  = (int)axvp->axv_trw;
		axvs->axv_ctrw = (int)axvp->axv_ctrw;
		axvs->axv_gain = (int)((axvp->axv_softcsr & 0x000c) >> 2);
		axvs->axv_poll = (int)(axvp->axv_state & AXV_POLL);
		axvs->axv_busy = (int)(axvp->axv_state & AXV_BUSY);
		axvs->axv_open = (int)(axvp->axv_state & AXV_OPEN);
		axvs->axv_read = (int)(axvp->axv_state & AXV_READ);
		return (0);
		}
	/*
	 * Make sure the device is opened and not busy before
	 * it is changed or its state is changed.
	 */
	if ((axvp->axv_state & AXV_BUSY) || !(axvp->axv_state & AXV_OPEN))
		return(EACCES);
	
	switch (cmd) 
		{
		/*
		 * Set the input channel (0..15)
		 */
		case ADIOSCHAN:
			{
			index0 = (* (int *) data);
			if ((index0 < 0) || (index0 > 16))
				return(EINVAL);
			axvp->axv_softcsr |=(short int)(index0 << 8);
			break;
			}
		/*
		 * Set the gain.
		 * The setting to voltage range mapping is: 
		 * 0   -10v to +10 (the default setting)
		 * 1   -5v to +5v
		 * 2   -2.5v to +2.5v
		 * 3   -1.25v - +1.25v
		 */
		case ADIOSGAIN:
			{
			index0 = (* (int *) data);
			if ((index0 < 0) || (index0 > 3))
				return(EINVAL);
			axvp->axv_softcsr &= (short int)~(0xc) ;
			axvp->axv_softcsr |= ((short int) index0 << 2);
			break;
			}
		/*
		 * Set the output channel
		 */
		case ADIOSDAC:
			{
			index0 = (* (int *) data);
			if ((index0 < 0) || (index0 > 1))
				return(EINVAL);
			if ( index0 )
				axvp->axv_dac = &(axvaddr->axv_dacB);
			else
				axvp->axv_dac = &(axvaddr->axv_dacA);
			break;
			}
		/*
		 * Set the polling or interrupt driven
		 */
		case ADIOSPOLL:
			{
			index0 = (* (int *) data);
			if ((index0 < 0) || (index0 > 1))
				return(EINVAL);
			if ( index0 )
				axvp->axv_state  |= AXV_POLL;
			else
				axvp->axv_state  &= ~AXV_POLL;
			break;
			}
		/*
		 * Set driver to send data send and ready for data receive
		 * signals.
		 */
		case ADIOSRXREADY:
			{
			index0 = (* (int *) data);
			if ((index0 < 0) || (index0 > 1))
				return(EINVAL);
			if ( index0 )
				axvp->axv_state  |= AXV_RXREADY;
			else
				axvp->axv_state  &= ~AXV_RXREADY;
			break;
			}
		/*
		 * Read and write data *data number of times. This
		 * test needs external clock applied.
		 */
		case ADIOTEST:
			{
			dac = axvp->axv_dac;
			spl6();
			axvaddr->axv_csr = axvp->axv_softcsr;
			index0 = 1000;
			while (index0-- > 0 &&((axvaddr->axv_csr & CSRMASK)!=axvp->axv_softcsr))
				axvaddr->axv_csr |= axvp->axv_softcsr;
			index1 = *(int *)data;
			index0 = 0;
			axvaddr->axv_csr |= AXV_SCHMITT;
			axvp->axv_state  |= AXV_BUSY;
			while (index0++ <= index1)
				{
				while( !(axvaddr->axv_csr & AXV_DONE))
				;
				*dac = axvaddr->axv_data;
				}
			axvp->axv_state &= ~AXV_BUSY;
			axvaddr->axv_csr &= ~AXV_SCHMITT;
			spl0();
			break;
			}
		/*
		 * Get a single word a data. Sample is initiated through
		 * program and not through external clock.
		 */
		case ADIOGETW:
			{
			spl6();
			axvp->axv_state |= AXV_BUSY;
			axvaddr->axv_csr = axvp->axv_softcsr;
			axvaddr->axv_csr |= AXV_START;
			while( !(axvaddr->axv_csr & AXV_DONE))
			;
			*(short int *)data = axvaddr->axv_data;
			axvp->axv_state &= ~AXV_BUSY;
			spl0();
			break;
			}
		/* 
		 * Put a single word to DAC. Output here does not rely on
		 * external clock as write does.
		 */
		case ADIOPUTW:
			{
			dac = axvp->axv_dac;
			spl6();
			axvaddr->axv_csr = axvp->axv_softcsr;
			axvp->axv_state  |= AXV_BUSY;
			*dac = *(short int *)data;;
			axvp->axv_state &= ~AXV_BUSY;
			spl0();
			break;
			}
		/*
		 * Not a legal ioctl command.
		 */
		default:
			return (ENOTTY);
		}
	return (0);
	}
/*-------------------------------------------------------------------*/

axvwrite(dev, uio)
	dev_t dev;
	struct uio *uio;
	{
	int 			error;
	register unsigned int	n;
	register short int  	dummy;
	register int 		index = 0;
	register short int 	*dac;
	
	register struct axv_softc 	*axvp = &axv[AXVUNIT(dev)];
	register struct axvdevice 	*axvaddr =
	    (struct axvdevice *) axvdinfo[AXVUNIT(dev)]->qi_mi->qm_addr;
	
	if ((unsigned)uio->uio_resid / 2  > BIGBUF)
		return( EINVAL );
	
	n = (unsigned)uio->uio_resid;
	
	error = uiomove((caddr_t)axvp->axv_bigbuf, (int)n, UIO_WRITE, uio);
	if (error)
		return(error);
	dac = axvp->axv_dac;
	/*
	 * Put the user data on the DAC register. Use the external 
	 * trigger set the DONE bit and effectively time the 
	 * output.
	 */
	spl6();
	if (axvp->axv_state && AXV_RXREADY)
		{
		axvrxready(axvaddr,512);
		dac = &(axvaddr->axv_dacA);
		}
	axvaddr->axv_csr = axvp->axv_softcsr;
	axvaddr->axv_csr |= AXV_SCHMITT;
	axvp->axv_state  |= AXV_BUSY;
	axvp->axv_state  &= ~AXV_READ;
	n 		= n / 2;
	axvp->axv_trw 	= n / 2;
	axvp->axv_ctrw 	= 0;
	if ( axvp->axv_state & AXV_POLL )
		{
		for ( index = 0; index <= n; index++ )
			{
			while( !(axvaddr->axv_csr & AXV_DONE))
			;
			dummy = axvaddr->axv_data;
			*dac = axvp->axv_bigbuf[index];  
			}
		axvaddr->axv_csr &= ~AXV_SCHMITT;
		axvp->axv_state  &= ~AXV_BUSY;
		axvp->axv_state  |= AXV_READ;
		}
	else
		{
		axvaddr->axv_csr |= AXV_IENABLE;
		while ( axvp->axv_state & AXV_BUSY )
			sleep(( caddr_t)axvp, AXVWAITPRI);
		}
	if (axvp->axv_state && AXV_RXREADY)
		axvrxready(axvaddr,0);
	spl0();
	return (0);
	}

/*-------------------------------------------------------------------*/
/*
 * Read fills axvp->axv_bigbuf with 16 bit word of data and then
 * calls uiomove to move the data down from kernel to user space.
 * The user specifies the number of bytes of data in the system call
 * to read, but  the actual almount is this number divided by two.
 * The buffer may be filled with a polling loop or interrupt driven
 * routines.
 */ 
axvread(dev, uio)
	dev_t dev;
	struct uio *uio;
	{
	int 				error = 0;
	register unsigned int		n;
	register struct axv_softc 	*axvp = &axv[AXVUNIT(dev)];
	register struct axvdevice 	*axvaddr =
	    (struct axvdevice *) axvdinfo[AXVUNIT(dev)]->qi_mi->qm_addr;
	
	if (uio->uio_resid / 2  > BIGBUF)
		return(EINVAL);
	
	axvp->axv_state  |= AXV_BUSY;
	axvp->axv_state  |= AXV_READ;
	axvp->axv_trw     = (int) uio->uio_resid /  2;
	axvp->axv_ctrw    = 0;
	n               = (unsigned)uio->uio_resid;
	
	if (axvp->axv_state & AXV_POLL )
		axvpollfill(axvaddr,axvp);
	else
		axvinterfill(axvaddr,axvp);
	
	error = uiomove((caddr_t) axvp->axv_bigbuf, (int)n, UIO_READ, uio);
	axvp->axv_state  &= ~AXV_BUSY;
	return(error);
	}

/*
 * This routine polls for axvp->ad_trw words of data.
 * An external clock must be used.
 * First set the clock bit on the csr, and then loop.
 * In the loop, poll the done bit in the csr to be set, get the
 * data, and continue.
 */
axvpollfill(axvaddr,axvp)
	register struct axvdevice 	*axvaddr;
	register struct axv_softc 	*axvp;
	{
	register int			 index0;
	
	spl6();
	if (axvp->axv_state && AXV_RXREADY)
		axvrxready(axvaddr,512);
	axvaddr->axv_csr = axvp->axv_softcsr;
	index0 = 1000;
	while (index0-- > 0&&((axvaddr->axv_csr & CSRMASK)!=axvp->axv_softcsr)) 
		axvaddr->axv_csr = axvp->axv_softcsr;
	axvaddr->axv_csr |= AXV_SCHMITT;

	/* This is a test, change this section */	
	while (axvp->axv_bigbuf[0] >= 2040)
		{
		while( !(axvaddr->axv_csr & AXV_DONE))
		;
		axvp->axv_bigbuf[0] = axvaddr->axv_data;
		}

	/* Poll and read the data register */
	index0 = 1;
	while (index0 < axvp->axv_trw)
		{
		while( !(axvaddr->axv_csr & AXV_DONE))
		;
		axvp->axv_bigbuf[index0++] = axvaddr->axv_data;
		}
	axvaddr->axv_csr &= ~AXV_SCHMITT;
	
	if (axvp->axv_state && AXV_RXREADY)
		axvrxready(axvaddr,0);
	spl0();
	return;
	}

/*
 * This routine sets up the interrupt driven read. First
 * force and interrupt and the sleep. Wakeup after the 
 * interrupt routine has executed axvp->axv_trw times.
 * Since the clock bit is set to initiate reads, an
 * external clock is again necessary.
 */
axvinterfill(axvaddr,axvp)
	register struct axvdevice 	*axvaddr;
	register struct axv_softc 	*axvp;
	{
	register int			 index0;
	
	spl6();
	if (axvp->axv_state && AXV_RXREADY)
		axvrxready(axvaddr,512);
	axvaddr->axv_csr = axvp->axv_softcsr;
	index0 = 1000;
	while (index0-- > 0&&((axvaddr->axv_csr & CSRMASK)!=axvp->axv_softcsr)) 
		axvaddr->axv_csr = axvp->axv_softcsr;
	axvaddr->axv_csr |= AXV_SCHMITT | AXV_IENABLE;
	while ( axvp->axv_state & AXV_BUSY )
		sleep(( caddr_t)axvp, AXVWAITPRI);
	axvaddr->axv_csr &= ~AXV_SCHMITT;
	if (axvp->axv_state && AXV_RXREADY)
		axvrxready(axvaddr,0);
	spl0();
	return;
	}

/*
 * Sends send ready to receive / ready to send signal thru dac B
 * Note : resets gain to 0
 */
axvrxready(axvaddr,v)
	register struct axvdevice *axvaddr;
	short int v;
	{
	axvaddr->axv_csr &= (short int) ~(0xc);
	axvaddr->axv_dacB = v;
	}
/*-------------------------------------------------------------------*/
axvintr(dev)
	dev_t dev; 
	{
	register struct axvdevice *axvaddr =
		(struct axvdevice *) (axvdinfo[AXVUNIT(dev)]->qi_mi)->qm_addr;
	register struct axv_softc *axvp = &axv[AXVUNIT(dev)];
	
	short int dummy = 0;
	
	if ( axvp->axv_ctrw <= axvp->axv_trw )
		if (axvp->axv_state & AXV_READ )
			{
			while( !(axvaddr->axv_csr & AXV_DONE))
			;
			axvp->axv_bigbuf[axvp->axv_ctrw++] = axvaddr->axv_data;
			axvaddr->axv_csr |=  AXV_IENABLE;
			}
		else
			{
			while( !(axvaddr->axv_csr & AXV_DONE))
			;
			dummy = axvaddr->axv_data;
			*(axvp->axv_dac) = axvp->axv_bigbuf[axvp->axv_ctrw++];  
			axvaddr->axv_csr |=  AXV_IENABLE;
			}
	else
		{
		axvp->axv_state  &= ~AXV_BUSY;
		axvp->axv_state  |= AXV_READ;
		axvp->axv_ctrw = 0;
		axvaddr->axv_csr &= ~AXV_SCHMITT;
		dummy = axvaddr->axv_data;
		wakeup((caddr_t)axvp);
		}
	
	}
/*-------------------------------------------------------------------*/

axvreset(qban)
	int qban;
	{
	register int i;
	register struct qb_device *qi;
	register struct axv_softc *axvp = axv;
	register struct axvdevice *axvaddr;

	for (i = 0; i < NAXV; i++, axvp++) 
		{
		if((qi = axvdinfo[i]) == 0 || qi->qi_alive == 0 ||
				!(axvp->axv_state & AXV_OPEN))
			continue;
		printf(" axv%d", i);
		if (axvp->axv_state & AXV_BUSY == 0)
			continue;
		axvaddr = (struct axvdevice *) qi->qi_mi->qm_addr;
		axvaddr->axv_csr = 0;
		axvaddr->axv_csr = axvp->axv_softcsr|AXV_IENABLE|AXV_START;
		}
	}
/*-------------------------------------------------------------------*/
#endif
