/*
 *	axvreg.h	1.0 (Milwaukee) 10/14/86
 */

/*
 * The registers on an AXV consist of a CSR, a data
 * buffer register, and a two data to analog conversion (DAC)
 * registers. All the registers are 16 bits but both the buffer
 * registers and the DAC's have only 12 significant digits. So
 * what ever user selected form the data is in (binary, offset binary,
 * or 2's complement), the range is 2048. The following is a summation
 * the registers.
 *
 */

/*
 * Control and Status Register (CSR)
 * 
 *      15  14  13  12  11  10   9   8   7   6   5   4   3   2   1   0
 *    -----------------------------------------------------------------
 *    |R/W|R/W| * | * |R/W|R/W|R/W|R/W| R |R/W|R/W|R/W|R/W|R/W| * | W |
 *    -----------------------------------------------------------------
 *      |   |           |   |   |   |   |   |   |   |   |   |       |
 *      |   |           -------------   |   |   |   |   -----       |
 *      |   |                  |        |   |   |   |     |         |
 *      |   |           Transmit Line   |   |   |   | Gain Select   |
 *      |   |                           |   |   |   |               |   
 *      | Error Int. Enable             |   |   |   |            A/D Start
 *      |                               |   |   |   |
 *    Error                             |   |   |  Ext. Trigger Enable 
 *                                      |   |   |
 *                                      |   |  RTC Enable 
 *                                      |   | 
 *                                      |  Done Int. Enable 
 *                                      |  
 *     *Unused                         A/D Done 
 *
 *
 */
 /*
  * To initiate a program start, the A/D bit is set.
  */
#define AXV_START		01

/*
 * Ext. Trigger Enable - When this is set, an external signal
 * from either Schmitt clock or any external trigger will initiate 
 * an analog to digital conversion. The source of the trigger is 
 * user selectable and may originate from either the Q-bus event line
 * or the pin 19 on the I/O connector. This driver assumes there
 * is a trigger signal and it is from the I/O connector.
 */
#define AXV_SCHMITT		020

/*
 * Real Time Clock Enable - A stream of sampling may be initiated
 * by setting the this bit. While set, each read of the data register
 * initiates another read. The clock signal line is pin 21 on the
 * A cable connector. This seems similiar to an external trigger
 * from the I/O connector.
 */
#define AXV_CLOCK		040
#define AXV_IENABLE		0100

/*
 * Interrrupt when analog to digital conversion is done.
 */
#define AXV_DONE 		0200

/*
 * When set, interrupt on error.
 */
#define AXV_INCENABLE		040000

/*
 * When set this bit indicates an error has occured. The
 * possible sources of error include:
 *  - Trying an external start or clock start during multiplexor
 *    settling time.
 *  - Trying a start while an A/D conversion in progress.
 *  - Trying a start while the A/D done bit is set.
 *  This bit is cleared by writing to the CSR.
 */
#define AXV_ERROR		0100000

 /*
  * Data Buffer Register:
  * The data buffer register is read only register that hold the digital
  * data after the A/D conversion is complete. The DBR can be read
  * aftr the A/D Done bit is set in the CSR.
  *
  *  Bits 0-11  - The analog to digital conversion data is store here
  *		  in one of the following data notations:
  *		    - binary
  *		    - offset binary
  *		    - 2's complement
  *		  The user selects data notation thru hard wired junpers.
  *
  *  Bits 12-15 - Sign for data in 2's complement form. Unused for
  *		  data in offset binary or bipolar form.
  */

 /*
  * DAC A and DAC B Registers
  * DAC A and DAC B are 16-bit write only registers. The are loaded from
  * the Q-bus with data to be changed to an analog voltage. Both
  * DAC's respond immediately to the data put in its register.
  * Each register holds its last value until the it is written 
  * again or until the power is turned off.
  * The format of the bits are the same as the data buffer register.
  */

struct axvdevice 
	{
	short int axv_csr;			/* Control status register */
	short int axv_data;			/* Data buffer             */
	short int axv_dacA;                      /* DAC A                   */
	short int axv_dacB;                      /* DAC B                   */
	};
	 

/*
 * Return status of hardware and software interface via ioctl.
 * Used for debugging.
 */
struct axvstatus
	{
	int axv_chan;
	int axv_dac;
	int axv_ctrw;
	int axv_trw;
	int axv_gain;
	int axv_busy;
	int axv_open;
	int axv_poll;
	int axv_read;
	};

/*
 * The ioctl constants.
 */
#define AD_SCHAN	ADIOSCHAN		/* set read channel   */
#define AD_GSTATUS	ADIOGSTATUS		/* get axv status */
#define AD_SDAC		ADIOSDAC 		/* set write channel */
#define AD_SGAIN        ADIOSGAIN		/* set gain            */
#define AD_SPOLL	ADIOSPOLL		/* set poll or interrupt */
#define AD_TEST		ADIOTEST		/* test read/write */
#define AD_GETW		ADIOGETW		/* get one sample */
#define AD_PUTW 	ADIOPUTW		/* put one sample */
#define AD_SRXREADY 	ADIOSRXREADY		/* signal recorder for data */

#define	ADIOSCHAN	_IOW(a, 0, int)
#define	ADIOGSTATUS	_IOR(a, 1, struct axvstatus)
#define ADIOSDAC        _IOW(a, 2, int)         
#define	ADIOSGAIN	_IOW(a, 3, int)	
#define ADIOSPOLL	_IOW(a, 4, int)
#define ADIOTEST	_IOW(a, 5, int)
#define ADIOGETW 	_IOR(a, 6, short int)
#define ADIOPUTW 	_IOW(a, 7, short int)
#define ADIOSRXREADY	_IOW(a, 8, short int)

/*
 * The kernel buffer size in 16 bit words and in bytes
 */
#define BIGBUF		02000000
#define READMAX		(2 * BIGBUF)
