/*	dhvreg.h	1.0	5/23/86		*/

/*
	  The registers in a DHV are not aranged quite like
	a DH.   Instead, there are two fixed registers ( the
	control status and receive/transmit registers), and
	then eight blocks of registers, one block for each
	line, which are selected through the indirect field
	in the CSR, and which then appear following the CSR
	in the address space:

	Control status register		CSR	Base		R/W
	Receive buffer			RBUF	Base + 02	R
	Transmit character		TXCHAR	Base + 02	W
      ---------------------------------------------------------------
8 blks| Line parameter register		LPR	Base + 04	R/W |
      | Line status			STAT	Base + 06	R   |
1 for | Line Control register		LNCTRL	Base + 010	R/W |
each  | Transmit buffer address 1	TBUFF1	Base + 012	R/W |
line  | Transmit buffer address 2	TBUFF2	Base + 014	R/W |
      | Transmit buffer count		TBUFFCT	Base + 016	R/W |
      ---------------------------------------------------------------
Selected thru indirect field in CSR.
*/

/*
	DHV device register definitions.
*/

struct	dhv_device {
	short	csr;		/* control status register */
	union	{
		/* The next two registers occupy the same address space.
		   A write to the address causes the transmit register
		   to be accessed; whereas a read from the address accesses
		   the receive regsiter.
		*/	
		short	rbuf;		/* receive character register accessed
					   on a read */
		short	txchar;		/* transmit character register accessed
					   on a write */
	} un;
	short	lpr;			/* line parameter register */
	u_short	stat ;			/* line status register */
	u_short	lnctrl;			/* line control register */
	u_short	tbuffad1;		/* transmit buffer address 1 */
	union { 			/* transmit buffer address 2 */
		u_short	tbuffad2;
		char	tbuffad2_ls[2];	/* For accessing only low byte */
#define	TB_LS	1 			/* For 68k machines -- access to low*/
	} unn;
	short	tbuffct;		/* buffer count register */
};
/*  */
/* Control and Status Resgister (CSR)-

CSR (BASE)

  15  14    13  12  11  10  9   8   7    6     5    4    3     2     1     0
-------------------------------------------------------------------------------
| R | R/W | R | R | R | R | R | R | R | R/W | R/W |   | R/W | R/W | R/W | R/W |
-------------------------------------------------------------------------------
 |     |    |   |   |   |   |   |   |    |     |         |     |     |     |
 |     |    |   |   -------------   |    |     |	 -------------------
 |     |    |   |         |         |    |     |		  |
 |     |    |   | Transmit line No. |    |     |  Indirect Address Reg Pointer
 |     |    |   |                   |    |     |  	(Channel No.)
 |     |    | Transmit DMA error    |    | Master Reset
 |     |    |                       |    |
 |     | Diagnostics failure        | Receive Int. Enable(RXIE)
 |     |                            | 
 | Transmit Int. Enable   Receive data available
 |
Transmitter Action

/* Transmitter Action - TX.ACTION (RD)
   	This bit is set by DHV11 when:

   	  1. The last character of a DMA buffer has left the DUART.
	  2. A DMA transfer has been aborted.
	  3. A DMA transfer has been terminated by the DHV11 because
	     of nonexistent memory being addressed, or because of a
	     memory parity error.
	  4. When a single-character programmed output has been
	     accepted.  That is to say, the character has been taken from
	     TX.BUFF.

	This bit is cleared when the CSR is read by the host.

	Also cleared by MASTER.RESET

	NOTE:  CSR contents should only be changed by a MOV or MOVB
	instruction.  Other instructions may lose the state of the
	TX.ACTION bit (CSR<15>).
*/
#define	DHV_TX_ACTION	0100000

/* Transmit Interrupt Enable - TXIE (R/W)
	When set, allows the DHV11 to interrupt the host when CSR<15>
	(TX.ACTION) becomes set.

	Cleared by BINIT but not by MASTER.RESET
*/
#define	DHV_TXIE		0040000		/* transmit interrupt enable */

/* Diagnostic Fail - DIAG.FAIL (RD)
	When set, indicated that DHV11 internal diagnostics have dectected
	an error.  The error may have been detected by the self-test dianostic
	or by the BMP.

	This bit is associated with the diagnostic-passed LED.  When it is set,
	the LED will be off.  When it is cleared, the LED will be on.

	The bit is set by MASTER.RESET.  It is cleared after internal
	diagnostic programs have been run successfully.

	It is only valid after the MASTER.RESET bit CSR<5> has been
	cleared.
*/
#define	DHV_DIAG_FAIL	0020000		/* diagnostic fail */

/* Transmit DMA error - TX.DMA.ERROR (RD)
	If set with TX.ACTION also set, means that the channel indicated
	by CSR<11:8> TX.LINE has failed to transfer DMA data within 10.7
	microseconds of the bus request being acknowledged, or that there is
	a memory parity error.
*/
#define	DHV_TX_DMA_ERROR	0010000		/* transmit DMA error */

/* Transmit Line Number - TX.LINE (RD)
	If TX.ACTION is set, these bits hold the binary number of the
	channel which has just:

	  1. Completed a DMA block transfer.
	  2. Accepted a single character for transmission.
	  3. Aborted a DMA block transfer.
	
	if TX.DMA.ERR is also set, these bits contain the binary number of
	the channel which hash failed during a DMA transfer.
*/
#define	DHV_TX_LINE	0007400		/* transmitter line no */

/* Received Data Available- RX.DATA.AVAIL (RD)
	When set, indicates that a received character is available.  This bit
	is clear when the FIFO is empty.  It is used to request an RX 
	interrupt.
*/
#define	DHV_RX_DATA_AVAIL	0000200		/* receiver data available */

/* Receiver Interrupt Enable - RXIE (R/W)
	When set, this bit allows the DHV11 to interrupt the host when
	RX.DATA.AVAIL is set.  An interrupt is generated under the
	following conditions:

	  1. RXIE is set and a character is placed into an empty FIFO.
	  2. The FIFO is not empty and RXIE is changed from 0 to 1.
	
	Cleared by BINIT but not by MASTER.RESET.
*/
#define	DHV_RXIE		0000100		/* receiver interrupt enable */

/* Master Reset - MASTER.RESET (R/W)
	Set by the host, in order to reset DHV11.  Stays set while DHV11
	runs a self-test diagnostic, and then performs an initialization
	sequence.  The bit is then cleared to tell the host that the process
	is complete.

	This bit is set by BINIT (bus initialization signal), or by the host
	procesor setting CSR<5>.

	The host should not write to this bit when it is already set.
*/
#define	DHV_MASTER_RESET	0000040		/* master reset */

/* Indirect address register - IND.ADDR.REG (R/W)
	These bits are used to select the wanted channel register when
	accessing a block of indexed (M) registers.  They form the binary
	number of the channel which is accessed.
*/
#define	DHV_IND_ADDR_REG	0000017		/* indirect address field */


#define	DHV_IE  (DHV_TXIE | DHV_RXIE)	/* Interrupt enable (TX & RX) */

/*  */
/* Receive Buffer (RBUF)
	- This register has the same address as the Transmit Character
	register(TXCHAR).  However, a READ from 'base+2' in interpreted
	by the DHV11 hardware as a READ from the FIFO.  Therefore, RBUF
	is a 256-character register with a single-word address.  The
	Least Significant Bit (LSB) of the character is in bit 0.

  RBUF (read BASE+2)

      *   *   *                   *   *   *   *   *   *   *   *
 15  14  13  12  11  10   9   8   7   6   5   4   3   2   1   0
-----------------------------------------------------------------
| R | R | R | R | R | R | R | R | R | R | R | R | R | R | R | R |
-----------------------------------------------------------------
  |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |
  |   |   |   |   -------------   -----------------------------
  |   |   |   |         |                       |
  |   |   |   | Receive Line No.   Received Character
  |   |   |   |				or
  |   |   | Parity error           Data set flags (from high byte of stat)
  |   |   | 				or
  |   | Framing Error		   Diagnostic Innformation
  |   |
  | Overrun Error
  |
Data Valid
   
/* bits in receive buffer */

/* Data Valid - DATA.VALID (RD)
	Set if the FIFO is not empty.  Cleared by MASTER.RESET or by
	the FIFO becoming empty.

	After self-test, diagnostic information if loaded into the FIFO.
	Therefor this bit is always set after a sccuessful master reset
	sequence.
*/
#define	DHV_DATA_VALID	0100000		/* receive data valid */

/* Overrun error - OVERRUN.ERR (RD)
	Set if one or more previous character of the channel indicated by
	bits RBUF<11:8> (RX.LINE) were lost because of a full FIFO or
	failure to service the UARTs (also see RX.CHAR).

	NOTE:  The 'all 1s' code for bits RBUF<14:12> is reserved.  This
	code indicates that modem or diagnostic information is held in
	RBUF<7:0> (RX.CHAR).
*/
#define	DHV_OVERRUN_ERR	0040000		/* overrun error */

/* Framing error - FRAME.ERR (RD)
	Set if the first stop bit if the received character was not
	detected (also see RX.CHAR).
*/
#define	DHV_FRAME_ERR	0020000		/* frame error */

/* Parity error - PARITY.ERR (RD)
	Set if this character has a parity error and parity is enable for
	the channel indicated by bits RBUF<11:8> (RX.LINE)
	(also see RX.LINE).
*/
#define	DHV_PARITY_ERR	0010000		/* parity error */

/* Receive line number - RX.LINE (RD)
	These bits hold the binary number of the channel on which the
	character of RBUF<7:0> was received or on which a data set
	change was reported.
*/
#define	DHV_RX_LINE	0007400		/* receive line number */

/* Received Character - RX.CHAR (RD)
	If RBUF<14:12> = 000, these eight bit contain the oldest character
	in the FIFO.  The character is good.

	If RBUF<14:12> = 001, 010, or 011, these eight bit contain the
	oldest character in the FIFO.  The character is bad.
	
	If RBUF<14:12> = 111, these eight bit contain diagnostic or
	modem status information.  In this case, RBUF<0> has the
	following meanings:

	  0 = Modem status in RBUF<7:1> (See line status register [STAT])
	  1 = Diagnostic information in RBUF<7:1>

	      D7  D6  D5  D4  D3  D2  D1  D0   Diagnostic status byte
	      |   |                   |   |
	      | if D7 = 1, then:      | 0 = Modem status code
	      | 0 = self-test code in | 1 = Diagnostic code
	      |     d5-d1	      |
	      | 1 = BMP code in d5-d1 |
	      |                     if D7 = 1, then:
     0 = Rom version in D6-D2,      0 = Proc1 specific errors in D4-D2
         D1 is the Proc No.         1 = Proc2 specific errors in D4-D2
     1 = Diagnostic code in D6-D1

	If there is an overrun condition, the UART data buffer for that
	channel will be cleared.  A null character, with RBUF<14> set,
	will be placed in the receive character FIFO.  The cleared data
	will be lost.

	The DHV11 does not have a break detect bit.  A line break is
	indicated to the program as a null character with the
	FRAME.ERR set.
*/
#define	DHV_RX_CHAR	0000377		/* received character or
					    data set status flags or
						diagnostic information */
#define DHV_C_DIAG	0000001		/* character received is diag info */
/*  */
/* Transmit character register (TXCHAR) - 
	Single-character programmed transfers are made via the transmit
	character register.  Bit function as follows:

  TXCHAR ( write BASE+2 )
   15  14  13  12  11  10   9   8   7   6   5   4   3   2   1   0
  -----------------------------------------------------------------
  | W |   |   |   |   |   |   |   | W | W | W | W | W | W | W | W |
  -----------------------------------------------------------------
    |                               |   |   |   |   |   |   |   |
  Transmit data valid               -----------------------------
						  |
					    Transmit Character
/* bits in tx_char */

/* Transmit data valid - TX.DATA.VALID (WR)
	When set, instructs the DHV11 to transmit the character held in bits

	TXCHAR<7:0>.  The bit is sensed by the DHV11 which then transfers the 
	character, clears the but, and sets TX.ACTION.
	
	TX.DATA.VALID and the character can be written together, or by
	separate MOVB instructions.
*/
#define	DHV_TX_DATA_VALID 0100000		/* transmit data valid */ 

/* Transmit Character - TX.CHAR (WR) 
	Character to be transmitted.  The LSB is bit 0.  For 7-,
	6-, or 5-bit characters, unused bits must be '0'.
*/
#define DHV_TX_CHAR	0377	/* Transmit character mask */
/*  */
/* Line Parameter Register (LPR) -
	This register is used to configure its associated channel.
	Bit function is as follows:
  
  LPR (BASE+4)
							  *   *
     15  14  13  12  11  10   9   8   7   6   5   4   3   2   1   0
    -----------------------------------------------------------------
    |R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|   |
    -----------------------------------------------------------------
      |   |   |   |   |   |   |   |   |   |   |   |   |   |   |
      -------------   -------------   |   |   |   -----   -----
	    |               |         |   |   |     |       |
   Transmit speed   Receive speed     |   |   |     | Dianostic code
				      |   |   |     |
				      |   |   | Character length
				      |   |   |
				      |   | Parity enable
				      |   |
				      | Even parity
				      |
				    Stop code
*/

/* bits in line parameter register */

/* Transmit data rate - TX.SPEED (R/W)
	Set to 1101 by MASTER.RESET (9600 bit/s).

	Defines the transmit data rate (see dh_speed_hack[] in dh.c)
*/
#define	DHV_TX_SPEED	0150000		/* transmit 9600 */

/* Receive data rate - RX.SPEED (R/W)
	Set to 1101 by MASTER.RESET (9600 bit/s).

	Defines the receive data rate (see dh_speed_hack[] in dh.c)
*/
#define	DHV_RX_SPEED	0006400		/* receive 9600 */

/* Stop code - STOP.CODE (R/W)
	Defines the length of the transmitted stop bit.

	0 = 1 stop bit for 5-, 6-, 7-, or 9-bit characters.
	1 = 2 stop bits for 6-, 7-, or 8-bit characters, or 1.5 stop bits
	    for 5-bit characters.
	
	Cleared by MASTER.RESET.
*/
#define	DHV_2STOP	0000200		/* two stop bits */

/* Even Parity - EVEN.PARITY (R/W)
	If LPR<5> (PARITY.ENAB) is set, this bit defines the type of
	parity.
		1 = Even parity.
		0 = Odd parity.
	
	Cleared by MASTER.RESET.
*/
#define	DHV_EVEN_PARITY		0000100		/* even parity */

/* Parity Enable - PARITY.ENAB (R/W)
	Parity enable. Causes a parity bit to be generated in transmit,
	and checked and stripped on receive.
		1 = Parity enabled.
		0 = Parity disabled.
	
	Cleared by MASTER.RESET.
*/
#define	DHV_PARITY_ENABLE	0000040		/* parity enable */

/* Character Length - CHAR.LGTH (R/W)
	Defines the length of characters.  Does not include start, stop
	and parity bits.

	00 = 5 bits
	01 = 6 bits
	10 = 7 bits
	11 = 8 bits

	Set to 11 by MASTER.RESET
*/
#define	DHV_8BITS	0000030		/* 8 bit characters */
#define	DHV_7BITS	0000020		/* 7 bit characters */
#define	DHV_6BITS	0000010		/* 6 bit characters */
#define	DHV_5BITS	0000000		/* 5 bit characters */

/* Diagnostic Code - DIAG (R/W)
	Diagnostic control codes.  Used by the host as follows:

	00 = Normal Operation.
	01 = Causes the Background Moniter Program (BMP) to report
	     the DHV11 status via the FIFO.
*/
#define	DHV_NORMAL	0000000		/* normal mode */
#define DHV_DIAG 	0000002		/* diagnostic mode */

#define	DHV_HDUPLX	0		/* we don't seem to have any
						way to set half duplex */
/*  */
/* Line Status Register (STAT) -
	The high byte of this register holds modem status information.
	The low byte is undefined.

  STAT (BASE+6)

   15  14  13  12  11  10   9   8   7   6   5   4   3   2   1   0
  -----------------------------------------------------------------
  | R |   | R | R | R |   |   | R |   |   |   |   |   |   |   |   |
  -----------------------------------------------------------------
    |       |   |   |           |
   DSR      |  DCD  |       Always 0
	    |       |
	    |      CTS
	    |
	RI (Ring Indicator)
*/

/* bits in line status register */

/* Data set ready - DSR (RD)
	Gives the resent status of the Data Set Ready (DSR) signal from the
	modem.

	  1 = ON
	  0 = OFF
	
	NOTE:  In order to report a change of modem status, the DHV11 writes
	the high byte of STAT into the low byte of RBUF.  RBUF<14:12> = 111
	to tell the host that RBUF<7:0> do not hold a received character.
*/
#define	DHV_DSR		0100000		/* data set ready */

/* Ring indicator - RI (RD)
	Gives the present status of the Ring Indicator (RI) signal from the
	modem.
	  1 = ON
	  0 = OFF
*/
#define	DHV_RI		0020000		/* ring indicator */

/* Data carrier detect - DCD (RD)
	Gives the present status of the Data Carrier Dectected (DCD) signal
	from the modem.
	  1 = ON
	  0 = OFF
*/
#define	DHV_DCD		0010000		/* data carrier detect */

/* Clear to send - CTS (RD)
	Gives the present status of the Clear To Send (CTS) signal from the
	modem.
	  1 = ON
	  0 = OFF
*/
#define	DHV_CTS		0004000		/* clear to send */

/* Status register, bit 8 - STAT (RD)
	Always 0 - Didn't you always need a zero?
*/
#define	DHV_STATSTAT	0000400		/* status bit - always 0.
						very useful */

/* Line Control Register (LNCTRL) -
	The main function of this register is to control the line interface.

  LNCTRL (BASE+10)

   15  14  13  12  11  10   9   8   7   6   5   4   3   2   1   0
  -----------------------------------------------------------------
  |   |   |   |R/W|   |   |R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|
  -----------------------------------------------------------------
		|           |   |   |   |   |   |   |   |   |   |
	       RTS         DTR  |   -----   |   |   |   |   | DMA abort 
				|     |     |   |   |   |   |
				|     |     |   |   |   | I Auto
				|     |     |   |   |   |
				|     |     |   |   | RX enable
				|     |     |   |   |
				|     |     |   | Break
				|     |     |   |
				|     |     | O Auto
				|     |     |
				|     | Force Xoff
				|     |
				| Maintenance Mode
				|
			    Link Type
*/


/* bits in line control register */

/* Request to send - RTS (R/W)
	This bit controls the Request To Send (RTS) signal.
	  1 = ON
	  0 = OFF
*/
#define	DHV_RTS		0010000		/* request to send */

/* Data terminal ready - DTR (R/W)
	This bit controls the Data Terminal Ready (DTR) signal.
	  1 = ON
	  0 = OFF
*/
#define	DHV_DTR		0001000		/* data terminal ready */

/* Link Type - LINK.TYPE (R/W)
	This bit must be set if the channel is to be connected to a modem.
	When this bit is set, any change in modem status will be reported
	via FIFO as well as the STAT register.

	If this bit is reset, this channel becomes a 'data leads only'
	channel.  Modem status information is loaded in the high byte of
	STAT but is not placed in the FIFO.
*/
#define	DHV_LINK_TYPE	0000400		/* link type */

/* Maintence Mode - MAINT (R/W)
	These bits can be written by the driver or test programs, in
	order to test the channel.

	The codeing is as follows:

	00 = Normal operation
	01 = Automatic echo mode - Received data is retransmitted
	     (regardless of the state of TX.ENA) at the data rate
	     selected for the receiver.  The receieved characters are
	     processed normally and placed in the received character
	     FIFO.  In this mode, the DHV11 will not transmit any
	     characters (this includes internally generated flow-control
	     characters).  The RX.ENA bit must be set when operating
	     in this mode.
	10 = Local loopback - The DUART channel output is internally
	     connected to the input.  Normal received data is ignored and the
	     transmit data line is held marking.  In this mode, flow-control
	     characters will be looped back instead of being transmitted.  The
	     data rate selected for the transmitter is used for both trans-
	     mission and reception.  The TX.ENA bit still controls trans-
	     mission in this mode.
	11 = Remote loopback - In this mode, received data is retransmitted
	     at a clock rate equal to the received clock rate.  The data is
	     not placed in the receiver FIFO.  The state of TX.ENA is
	     ignored.
*/
#define DHV_MAINT	000300	/* maintenance mode */

/* Force X-off - FORCE.XOFF (R/W)
	This bit can be set by the program to indicated that this channel
	is congested at the host system (for example, if the typeahead
	buffer is full).  When it sees this bit set, the DHV11 will send
	an X-off code.  Until the bit is reset, X-offs will be sent after
	every alternate character received on that channel.  When the bit
	is reset, an X-on will be set unless IAUTO is set and the FIFO is
	critical.
*/
#define	DHV_FORCE_XOFF	000040	/* force X-off */

/* Outgoing auto flow - OAUTO (R/W)
	This bit is the auto-flow control bit for outgoing characters.
	When set, if RX.ENA is also set, the DHV11 will automatically
	respond to X-On and X-Off codes received from a channel.  The
	DHV11 uses the TX.ENA bit in TBUFFAD2 to stop and start the flow.
*/
#define	DHV_OAUTO	000020	/* outgoing auto flow */

/* Break control - BREAK (R/W)
	Is set, this bit forces the transmitter of this channel to the
	spacing state.

	Transmission is restarted when the bit is cleared.

	NOTE:  There is a short delay between writing the bit and the
	       channel changing state.  The delay is dependent on
	       throughput.  Because of normal length of a BREAK signal,
	       this should not cause problems.
*/
#define	DHV_BREAK	0000010		/* force to spacing state */

/* Receiver enable - RX.ENA (R/W)
	If set, this receiever channel is enabled.

	If reset when this DUART channel is assembling a character, that
	character is lost.

	Cleared by MASTER.RESET
*/
#define	DHV_RX_ENA	0000004		/* receiver enable */

/* Incomming auto Flow - IAUTO (R/W)
	This is the quto-flow control bit for incoming characters.  If it
	is set, the DHV11 will control incoming characters by transmitting
	X-on and X-off codes.

	If the FIFO becomes congested, the DHV11 will send an X-off code
	to the channels with this bit set.  An X-on will be sent when the
	congestion is reduced.
*/
#define	DHV_IAUTO	0000002		/* incoming auto flow */

/* DMA Abort */
#define DHV_TX_DMA_ABORT 0000001


#define	DHV_ON		(DHV_DTR | DHV_RTS | DHV_RX_ENA)
#define	DHV_OFF		(DHV_LINK_TYPE | DHV_RX_ENA)
/*  */
/* Transmit Buffer Address Register Number 1 (TBUFFAD1) -

  TBUFFAD1 (BASE+12)

     15  14  13  12  11  10   9   8   7   6   5   4   3   2   1   0
    -----------------------------------------------------------------
    |R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|
    -----------------------------------------------------------------
      |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |
      -------------------------------------------------------------
				    |
			Transmit DMA address
			( Bit 0 -15 )

Transmit Buffer address [low] - TBUFFAD (R/W)
	Bits <15:0> of the DMA address
*/
/*  */
/* Transmit buffer address register number 2 (TBUFFAD2)

  TBUFFAD2 (BASE+14)
     15  14  13  12  11  10   9   8   7   6   5   4   3   2   1   0
    -----------------------------------------------------------------
    |R/W|   |   |   |   |   |   |   |R/W|   |R/W|R/W|R/W|R/W|R/W|R/W|
    -----------------------------------------------------------------
      |                               |       |   |   |   |   |   |
    Transmit Enable              DMA start    ---------------------
							|
						Transmit DMA address
						(Bits 16-21)
*/

/* bite in transmit buffer address register 2 */

/* Transmitter Enable - TX.ENA (R/W)
	When set, the DHV11 will transmit all charaters.

	When cleared, the DHV11 will only transmit internally generated
	flow-control characters.

	Set by MASTER.RESET.

	In the OAUTO mode, this bit is used by the DHV11 to control
	outgoing characters.
*/
#define	DHV_TX_ENA	0100000		/* enable transmitter */

/* Transmit DMA start - TX.DMA.START (R/W)
	Set by the host to start a DMA transfer.  The DHV11 will reset the
	bit before returning TX.ACTION.

	Cleared by MASTER.RESET

	NOTE:  After setting this bit, the hist must not write to TBUFFCT,
	TBUFFAD1, or TBUFFAD2<7:0> until the TX.ACTION report has been
	returned.
*/
#define	DHV_DMA_START	0000200		/* start a DMA transfer */

/* Transmit Buffer address [high] - TBUFAD<21:16> (R/W)
	Bits <21:16> of the DMA address.

	Before a DMA transfer, the TBUFFAD1 and the low byte of TBUFFAD2
	are loaded with the start address of the DMA buffer.  This address
	is not valid during a DMA transfer.  When TX.ACTION is returned, the
	address will be valid
*/
#define	DHV_TBUFFAD2	000077
/*  */
/* Transmit DMA buffer counter (TBUFFCT) -

  TBUFFCT (BASE+16)

     15  14  13  12  11  10   9   8   7   6   5   4   3   2   1   0
    -----------------------------------------------------------------
    |R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|R/W|
    -----------------------------------------------------------------
      |   |   |   |   |   |   |   |   |   |   |   |   |   |   |   |
      -------------------------------------------------------------
				    |
			DMA character count
	(When valid, hold the number of chars, still to be sent)

*/

/* Transmit character count - TX.CHAR.CT (R/W)
	Loaded with the number of characters to be transferred by DMA.

	The number of characters is specified as a 16-bit unsigned integer.

	After a DMA transfer has been aborted, this location will hold the
	number of characters still to be transfered.

	See NOTE under TX.DMA.START
*/

